/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozcc
 *
 * The Initial Developer of the Original Code is
 * Nathan R. Yergler.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Nathan R. Yergler <nathan@yergler.net>
 *
 * ***** END LICENSE BLOCK ***** */

/* 
 * $Id: prefs.js,v 1.1 2004/05/18 16:23:08 nyergler Exp $
 */

var _elementIDs = ["pref_demo"];
var PREFS_FILE  = "mozcc_prefs.js";

// loadAppPrefs
//   Loads the application specific preferences file from the current
//   profile; constant PREFS_FILE contains the filename of the prefs file.
//   Returns nsIPrefBranch.
function loadAppPrefs() {

	// get the current profile directory
	var profileMgr = Components.classes["@mozilla.org/profile/manager;1"].getService(Components.interfaces.nsIProfile);

	var profileDtl = Components.classes["@mozilla.org/profile/manager;1"].getService(Components.interfaces.nsIProfileInternal);

	var currProfile = profileMgr.currentProfile;
	var profileDir = profileDtl.getProfileDir(currProfile);

	var dirEntries = profileDir.directoryEntries;
	while (dirEntries.hasMoreElements()) {
		prefsFile=dirEntries.getNext()
		prefsFile.QueryInterface(Components.interfaces.nsIFile, prefsFile);
		if (prefsFile.leafName == PREFS_FILE) {
		   break;
		}
	} // while has more elements

	// read the mozcc preferences file
	var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
	prefService.readUserPrefs(prefsFile);
	var prefs = prefService.getBranch("");
	
	return prefs;

} // loadAppPrefs

// initPrefs
//   Initializes the preferences panel to the current values
function initPrefs(event) {

      var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

      // initialize the default window values...
      for( var i = 0; i < _elementIDs.length; i++ )
      {        
        var elementID = _elementIDs[i];

        var element = document.getElementById(elementID);
        if (!element) break;
        var eltType = element.localName;

        if (eltType == "radiogroup")
          element.selectedItem = element.childNodes[pref.getIntPref(element.getAttribute("prefstring"))];
        else if (eltType == "checkbox")
          element.checked = pref.getBoolPref(element.getAttribute("prefstring"));
        else if (eltType == "textbox")
          element.setAttribute("value", pref.getCharPref(element.getAttribute("prefstring")) );
      }
}

    function savePrefs(event)
    {
      var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

      for( var i = 0; i < _elementIDs.length; i++ )
      {
        var elementID = _elementIDs[i];

        var element = document.getElementById(elementID);
        if (!element) break;
        var eltType = element.localName;

        if (eltType == "radiogroup")
          pref.setIntPref(element.getAttribute("prefstring"), parseInt(element.value));
        else if (eltType == "checkbox")
          pref.setBoolPref(element.getAttribute("prefstring"), element.checked);
        else if (eltType == "textbox")
          pref.setCharPref(element.getAttribute("prefstring"), element.value);
       }
    }

