/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozCC
 *
 * The Initial Developer of the Original Code is
 * Nathan R. Yergler.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Nathan R. Yergler <nathan@yergler.net>
 *
 * ***** END LICENSE BLOCK ***** */

/* mozcc.js
 * $Id: mozcc.js,v 1.1 2004/06/25 13:25:01 nathan Exp $
 */

// include("chrome://mozcc/content/ccrdf.js");
include(jslib_remotefile);
include(jslib_prefs);

const pageLoaderIface = Components.interfaces.nsIWebPageDescriptor;
const nsISelectionPrivate = Components.interfaces.nsISelectionPrivate;
const nsISelectionController = Components.interfaces.nsISelectionController;
var gBrowser = null;
var gViewSourceBundle = null;
var gPrefs = null;

var gLastLineFound = '';
var gGoToLine = 0;

var parsed = null;

try {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  gPrefs = prefService.getBranch(null);
} catch (ex) {
}

var gSelectionListener = {
  timeout: 0,
  notifySelectionChanged: function(doc, sel, reason)
  {
    // Coalesce notifications within 100ms intervals.
    if (!this.timeout)
      this.timeout = setTimeout(updateStatusBar, 100);
  }
}

// addLoadListener:
//   Retrieves a handle to the browser window and connects to 
//   the load events for catching location changes

function addLoadListener() {

  // listen for navigation events
  getBrowser().addEventListener("load", onLoadContent, true);

  // listen for tab switches
  getBrowser().addEventListener("select", onLoadContent, true);

} // addLoadListener


function getButtonVisibility() {

  var pref = new Prefs();
  return pref.getBool('mozcc.toolbar.hidden');

}

function delayedParse(rdf) {

  uri = _content.document.URL;
  index = window.parsed.push(new ccRdf()) - 1;

  window.parsed[index].parse(rdf, uri);
  updateStatusBar([window.parsed[index]]);

} // delayedParse

function delayedRetrieval(url) {

  // retrieve the URL and schedule the parsing
  window.setTimeout(delayedParse, 0, retrieveRdf(url));

} // delayedRetrieval

// extractRDF:
//   Extracts any RDF blocks from [textblock]; returns an array of strings

function extractRdf(curr_doc) {

  // check for embedded RDF
  rdf_regex = /<rdf:rdf[\w\W]*?<\/rdf:rdf>/i

  var result = new Array();
  var inputstr = "";

  // serialize the document so we can scan for RDF
  var serializer = new XMLSerializer();
  inputstr = serializer.serializeToString(curr_doc.documentElement);

  match = "";

  // extract any embedded RDF
  while (match != null) {
     match = inputstr.match(rdf_regex);

     if (match != null) {

	// schedule the delayed parsing
	window.setTimeout(delayedParse, 0, match[0]);

        // result[result.length] = match[0]; 

        inputstr = inputstr.substr(inputstr.search(rdf_regex) + match[0].length, inputstr.length);
     } // if match is not null

  } // while match is not null

  // retrieve any linked RDF
  rdfLinks = extractLinks(curr_doc); 
  for (i=0; i < rdfLinks.length; i++) {

    // schedule the delayed retrieval
    window.setTimeout(delayedRetrieval, 0, rdfLinks[i]);

    // result[result.length] = retrieveRdf(rdfLinks[i]);
  } // for each link 

  // return result;

} // extractRdf

// extractLinks
function extractLinks(current_doc) {

  function makeAbsolute(current_doc, link_href) {

      // this is an RDF link; parse it
      if (link_href.indexOf('http://') == 0) {
         // absolute link with protocol; no massaging necessary
         return link_href;
      } else 
      if (link_href.charAt(0) == '/') {
         // absolute link; add the server
	abs_parts = current_doc.URL.split('/');

	// drop off everything after the server
	while (abs_parts.length > 3) {
	   abs_parts.pop();
	} // while more than the server
	abs_parts.push(link_href);

	return abs_parts.join('/');
      } else {
         // relative link
         abs_parts = current_doc.URL.split('/');

         // assemble the new link
         abs_parts.pop()
         abs_parts.push(link_href);

         return abs_parts.join('/');

      } // relative link

      // fall-through case (although we shouldn't be able to get here)
      return link_href;

  } // makeAbsolute

  var results = new Array();

  // ****
  // get <link rel="meta"> links from the head
 
  // get a reference to the head of the document to check for link tags
  head = current_doc.documentElement.getElementsByTagName('head');
  if (head.length > 0) {
     // has a head element
     links = head[0].getElementsByTagName('link');
  }
  
  for (i = 0; i < links.length; i++) {
    if (links[i].getAttribute('rel') == 'meta') {

      results.push(makeAbsolute(current_doc, links[i].getAttribute('href')));

    } // if the link is metadata

  }  // for each link

  // ****
  // get <a rel="license"> links from the body

  anchors = current_doc.documentElement.getElementsByTagName('a');

  for (i = 0; i < anchors.length; i++) {
     if (anchors[i].hasAttribute('rel')) {
        if (anchors[i].getAttribute('rel') == 'license') {
           results.push(makeAbsolute(current_doc, anchors[i].getAttribute('href')));
        } // if rel=license
     } // if the anchor has a rel="" attr...
  } // for each anchor

  return results;

}

function retrieveRdf(url) {

  var rf = new RemoteFile(url); 

  // make sure the linked file exists
  if (!(rf.exists())) return "";

  // open the file and read the contents (presumably rdf)
  rf.open();
  var contents = rf.read();

  // return the contents
  return contents

} // retrieveRdf

// onLoadContent:
//   Responds to load events; checks for RDF and updates the status bar and
//   toolbar button accordingly.

function onLoadContent(event) {

  // alert(parsed);
  // alert(window.parsed);

  if (window.pared == null) {
      window.parsed = new Array();
  } else {
      window.parsed = new Array();
  }

  // window.parsed = new Array();
  // var parsed = new Array();
  var i = 0;

  // get handles to some UI elements
  button = document.getElementById('cc-button');
  statuspanel = document.getElementById('cc-status');

  // reset everything to non-RDF
  clearStatusPanel(statuspanel);

  if (button != null) {
     // the button is actually in the toolbar

     if (window.toolbar.visible) {
        // only set the button to disabled if the toolbar is visible
        // otherwise the browser barfs
        button.disabled = true;
     }

     button.hidden = getButtonVisibility();

  } // if button is not null

  // get the RDF elements (if any)
  rdfBlocks = extractRdf(_content.document);

/*
  // if there are any embedded RDF blocks, parse them
  uri = _content.document.URL;
  for (i=0; i < rdfBlocks.length; i++) {
    index = parsed.push(new ccRdf()) - 1;

    parsed[index].parse(rdfBlocks[i], uri);
    updateStatusBar([parsed[index]]);

  } // for each embedded RDF block
*/

  return true;

} // onLoadContent

function createImage(src, tip) {
   var newimg = document.createElement("image");

   newimg.setAttribute("src", src);
   newimg.setAttribute("alt", tip);
   newimg.tooltipText = tip;

   return newimg;
}

function createDesc(value) {
   var newdesc = document.createElement("description");
   newdesc.setAttribute("value", value);

   return newdesc;
}

function createCont(parent) {
   // creates an HBox, appends it to the parent and returns a reference
   var newcont = document.createElement("hbox");
   parent.appendChild(newcont);
   return newcont;
} // createCont

function clearStatusPanel(panel) {
   // remove all children from the parameter [panel]
   while (panel.hasChildNodes()) {
      panel.removeChild(panel.firstChild);
   }

   // add the basic CC icon
   ccIcon = createImage('chrome://mozcc/skin/images/cc_status_bar_nolic.png', 
                        'Creative Commons');
   ccIcon.setAttribute("id", "cc_status_logo");

   panel.appendChild(ccIcon);
   
   return true;
} // clearStatusPanel

function makeLicenseImages(license, container, showDesc) {

         // check for public domain
         if (license.isPublicDomain()) {
            if (showDesc) {
               imgContainer = createCont(container);
            } else imgContainer = container;

            imgContainer.appendChild(
               createImage('chrome://mozcc/skin/images/pd', 'Public Domain'));
            if (showDesc) imgContainer.appendChild(
                                       createDesc('Public Domain'));
      
            return;
         } // if public domain

         // check for sampling licenses
         if (license.isSampling()) {
            if (showDesc) {
               imgContainer = createCont(container);
            } else imgContainer = container;

            imgContainer.appendChild(
               createImage('chrome://mozcc/skin/images/recombo.png', 
                           'Recombo'));
            if (showDesc) imgContainer.appendChild(
                                       createDesc('Recombo: Allows Sampling'));
         } // if is sampling
         else
         if (license.isSamplingPlus()) {
            if (showDesc) {
               imgContainer = createCont(container);
            } else imgContainer = container;

            imgContainer.appendChild(
               createImage('chrome://mozcc/skin/images/recombo.png', 
                           'Recombo+'));
            if (showDesc) imgContainer.appendChild(createDesc('Recombo+: Allows Sampling+'));
         } // if is sampling+
 	 else
	 if (license.isGPL()) {
            if (showDesc) {
               imgContainer = createCont(container);
            } else imgContainer = container;

            imgContainer.appendChild(
               createImage('chrome://mozcc/skin/images/gpl.png', 
                           'GPL 2.0'));
            if (showDesc) imgContainer.appendChild(createDesc('GPL 2.0'));
	 } // if is GPL
	 else
	 if (license.isLGPL()) {
            if (showDesc) {
               imgContainer = createCont(container);
            } else imgContainer = container;

            imgContainer.appendChild(
               createImage('chrome://mozcc/skin/images/lgpl.png', 
                           'LGPL 2.1'));
            if (showDesc) imgContainer.appendChild(createDesc('LGPL 2.1'));
	 } // if is LGPL

         // initialize the flag for no derivatives 
         // (since it's flagged by the lack of a permission, 
         //  not a requirement/prohibition)
         no_derivs = true;
	 notice = false;
	 attribution = false;

         // extract the properties and add them to the status bar 
         requires = license.getAll('http://web.resource.org/cc/requires');
         permits  = license.getAll('http://web.resource.org/cc/permits');
         prohibits = license.getAll('http://web.resource.org/cc/prohibits');

         for (p = 0; p < requires.length; p++) {
             // get the short target name
             objt = requires[p].Value.split('/').reverse()[0];

             switch(objt.toLowerCase()) {
               case 'notice': 
                 if (!attribution) {
                   if (showDesc) {
                      imgContainer = createCont(container);
                   } else imgContainer = container;

                   imgContainer.appendChild(
                      createImage('chrome://mozcc/skin/images/attrib',
                                  'requires ' + objt));
                   if (showDesc) imgContainer.appendChild(
                      createDesc('requires ' + objt));

                   notice=true;
                 }
                 break;

               case 'attribution': 
                 if (!notice) {
                   if (showDesc) {
                      imgContainer = createCont(container);
                   } else imgContainer = container;

                   imgContainer.appendChild(
                      createImage('chrome://mozcc/skin/images/attrib',
                                  'requires ' + objt));
                   if (showDesc) imgContainer.appendChild(
                      createDesc('requires ' + objt));

                   attribution=true;
                 }
                 break;
               case 'sharealike': 
                 if (showDesc) {
                    imgContainer = createCont(container);
                 } else imgContainer = container;

                 imgContainer.appendChild(
                    createImage('chrome://mozcc/skin/images/share',
                                'requires ' + objt));
                   if (showDesc) imgContainer.appendChild(
                      createDesc('requires ' + objt));
                 break;
               case 'sourcecode': 
                 if (showDesc) {
                    imgContainer = createCont(container);
                 } else imgContainer = container;

                 imgContainer.appendChild(
                    createImage('chrome://mozcc/skin/images/source.png',
                                'requires ' + objt));
                   if (showDesc) imgContainer.appendChild(
                      createDesc('requires ' + objt));
                 break;
             } // switch on requirement

         } // for each requirement

         for (p = 0; p < permits.length; p++) {
             // get the short target name
             objt = permits[p].Value.split('/').reverse()[0];

             switch(objt.toLowerCase()) {
                                   case 'reproduction': break;
                                   case 'distribution': break;
                                   case 'derivativeworks': 
                                        no_derivs = false;
                                        break;
             } // switch target
         } // for each permission

         for (p = 0; p < prohibits.length; p++) {
             // get the short target name
             objt = prohibits[p].Value.split('/').reverse()[0];

             switch(objt.toLowerCase()) {
                                   case 'commercialuse': 
                                        if (showDesc) {
                                          imgContainer = createCont(container);
                                        } else imgContainer = container;

                                        imgContainer.appendChild(
                                           createImage(
                                          'chrome://mozcc/skin/images/noncomm',
                                           'prohibits ' + objt));
                                        if (showDesc) imgContainer.appendChild(
                                           createDesc('prohibits ' + objt));
                                        break;
             } // switch prohibition
         } // for each prohibition

         if (no_derivs) {
            if (showDesc) {
               imgContainer = createCont(container);
            } else imgContainer = container;

            imgContainer.appendChild(
               createImage('chrome://mozcc/skin/images/nomod', 
                            'prohibits DerivativeWorks'));
               if (showDesc) imgContainer.appendChild(
                            createDesc('prohibits DerivativeWorks'));
         } // if no derivatives

} // makeLicenseImages

function updateStatusBar(args) {

   // get a handle to UI elements
   statuspanel = document.getElementById('cc-status');
   button = document.getElementById('cc-button');

   cc = args[0];
   works = cc.works();

   // the work reference for this page should only have one element
   if (works.length == 1) {
       licenses = works[0].licenses();
   } else if (works.length == 0) {
       licenses = cc.licenses();
   } else {
       return false;
   }

   // extract the license properties and update the status bar
   if (licenses.length < 1) { return false; }

   // there must be some license, so enable the button.
   if (window.toolbar.visible) {

      if (button) button.disabled = false;

      // update the status bar icon to non-crossed
      document.getElementById('cc_status_logo').setAttribute('src', 'chrome://mozcc/skin/images/cc_status_bar.png');
   }

   for (l = 0; l < licenses.length; l++) {
      makeLicenseImages(licenses[l], statuspanel);
   } // for each license

   return true;

} //updateStatusBar

// showLicense:
//   displays the license information dialog for RDF contained in [aDocument];
//   derived from "view page source" code in browser.js

function showLicense(aDocument) {

  // open a dialog displaying current license information
  openDialog("chrome://mozcc/content/license.xul", "Creative Commons License",
             "chrome,modal=no,dialog=no,resizable,titlebar,centerscreen", 
	     aDocument,window.parsed);

  return true;

} // showLicense

function getBrowser()
{
  if (!gBrowser)
    gBrowser = document.getElementById("content");
  return gBrowser;
}

